from logging import Logger

import rapidjson
from flask import Response, request, Flask
from hou_flask_psycopg2 import SQLNotFoundException
from ultra_config import GlobalConfig
from werkzeug.exceptions import HTTPException, default_exceptions

__all__ = ["FlaskErrorHandler"]


class FlaskErrorHandler:
    def __init__(self, logger: Logger):
        self.app = None
        self.login_url = None
        self.logger = logger

    def init_app(self, app: Flask, login_url: str) -> Flask:
        self.app = app
        self.login_url = login_url

        app.register_error_handler(
            SQLNotFoundException, self.handle_object_not_found_exception
        )
        app.register_error_handler(Exception, self.handle_generic_exception)
        for exception_type in default_exceptions.values():
            app.register_error_handler(exception_type, self.handle_http_exception)

        return app

    def handle_object_not_found_exception(
        self, exception: SQLNotFoundException
    ) -> Response:
        self._log_exception(exception, verbose=False)
        return self._build_error_response(
            status_code=404,
            message="The requested object either does not "
            "exist or you do not have access to it",
            error_class="ResourceNotFoundException",
        )

    def handle_generic_exception(self, exception: Exception) -> Response:
        """
        The base exception handler for arbitrary exceptions
        Returns 500 errors with no information to avoid security issues
        """
        self._log_exception(exception)
        return self._build_error_response()

    def handle_http_exception(self, exception: HTTPException):
        """
        Handle HTTPExceptions from werkzeug
        """
        status_code = getattr(exception, "code", 500)
        if status_code >= 500:
            self._log_exception(exception)
            return self._build_error_response(status_code=status_code)
        if status_code < 400:
            raise exception
        extra_params = {}
        if status_code == 401:
            extra_params["login_url"] = self.login_url

        self._log_exception(exception, verbose=False)
        return self._build_error_response(
            status_code=status_code,
            message=exception.description,
            error_class=self._get_exception_name(exception),
            **extra_params
        )

    @staticmethod
    def _build_error_response(
        status_code=500,
        message="An Error has Occurred",
        error_class="ServerError",
        **extra
    ):
        data = {"message": message, "error_type": error_class}
        data.update(extra)
        return Response(
            rapidjson.dumps(data), status=status_code, content_type="application/json"
        )

    @staticmethod
    def _get_exception_name(exception: Exception):
        if hasattr(exception, "name"):
            return exception.name
        return exception.__class__.__name__

    def _log_exception(self, exception: Exception, verbose=True):
        if verbose or GlobalConfig.config["VERBOSE_4XX_ERROR_LOGGING"]:
            self.logger.exception(exception)
        exception_name = self._get_exception_name(exception)
        self.logger.warning(
            'Failing on route "%s": %s\nHeaders: %s\nBody: %s',
            request.url,
            exception_name,
            request.headers,
            request.get_data(),
        )



"""
Unit tests intended to help test the meta endpoints of
the application: e.g. `/version` and `/status`
"""
import rapidjson

import oss_auth


class MetaFunctionalTest:
    url = "/v1/meta"

    def test_status(self, test_app):
        resp = test_app.get(f"{self.url}/status")
        assert resp.status_code == 200

    def test_version(self, test_app):
        resp = test_app.get(f"{self.url}/version")
        assert resp.status_code == 200
        data = rapidjson.loads(resp.body)
        assert data["version"] == oss_auth.__version__



from uuid import uuid4

import pytest
from flask import Flask, g

from oss_auth.daos.permission_dao import Permission, Role
from oss_auth.exceptions import (
    MissingPermissionsException,
    ParameterNotPresentException,
)
from oss_auth.utils.authorization import require_org_permissions

from ...utils.factories import AccountFactory


class RequireAccountOrgPermissionsTest:
    @require_org_permissions(Permission.create_department)
    def _my_func(self, organization_id=None):
        return organization_id

    def test_when_no_organization_id__raises_exception(self):
        with pytest.raises(ParameterNotPresentException) as exc_info:
            self._my_func()

        assert "{}._my_func".format(self.__module__) in str(exc_info.value)

    def test_when_account_missing_override_permissions_and_not_same_org__raises_exception(
        self
    ):
        account = AccountFactory()
        with Flask("blah").app_context():
            g._current_account = account.id
            with pytest.raises(MissingPermissionsException):
                self._my_func(organization_id=uuid4())

    def test_when_account_has_override_permissions__no_exception(self):
        account = AccountFactory(role_name=Role.superuser.value)
        with Flask("blah").test_request_context():
            g._current_account = account.id
            org_id = uuid4()
            resp = self._my_func(organization_id=org_id)
            assert resp == org_id



from oss_auth.daos.permission_dao import PermissionDAO, Role
from tests.utils.factories import (
    AccountFactory,
    DepartmentFactory,
    OrganizationAccountFactory,
)
from tests.utils.factories.department import DepartmentAccountFactory


class GetRolePermissionsTest:
    def test_when_no_role__returns_empty_set(self):
        resp = PermissionDAO.get_role_permissions(Role.no_role.value)
        assert resp == frozenset()


class GetAccountRolesForDepartmentTest:
    def test_when_not_in_org__returns_default_role_only(self):
        account = AccountFactory()
        dep = DepartmentFactory()
        resp = PermissionDAO.get_account_roles_for_department(account.id, dep.id)
        assert resp == {account.role_name}

    def test_when_in_org__returns_default_and_org_role(self):
        org_account = OrganizationAccountFactory()
        dep = DepartmentFactory(organization=org_account.organization)
        resp = PermissionDAO.get_account_roles_for_department(
            org_account.account_id, dep.id
        )
        assert resp == {org_account.account.role_name, org_account.role_name}

    def test_when_in_department__returns_department_org_and_default_roles(self):
        dep_account = DepartmentAccountFactory()
        resp = PermissionDAO.get_account_roles_for_department(
            dep_account.account_id, dep_account.department_id
        )
        assert resp == {
            dep_account.role_name,
            dep_account.organization_account.role_name,
            dep_account.account.role_name,
        }

    def test_when_in_parent_department__returns_parent_department_role(self):
        dep_account = DepartmentAccountFactory()
        child = DepartmentFactory(
            parent=dep_account.department, organization=dep_account.organization
        )
        resp = PermissionDAO.get_account_roles_for_department(
            dep_account.account_id, child.id
        )
        assert resp == {
            dep_account.role_name,
            dep_account.organization_account.role_name,
            dep_account.account.role_name,
        }

    def test_when_not_in_parent_department__returns_child_department_only(self):
        dep = DepartmentFactory()
        child = DepartmentAccountFactory(
            department__parent=dep, organization=dep.organization
        )
        resp = PermissionDAO.get_account_roles_for_department(
            child.account_id, child.department_id
        )
        assert resp == {
            child.role_name,
            child.organization_account.role_name,
            child.account.role_name,
        }

    def test_when_in_child_and_parent__returns_child_department_and_parent_department(
        self
    ):
        parent = DepartmentAccountFactory()
        child = DepartmentAccountFactory(
            department__parent=parent.department,
            organization=parent.organization,
            organization_account=parent.organization_account,
            account=parent.account,
        )
        resp = PermissionDAO.get_account_roles_for_department(
            child.account_id, child.department_id
        )
        assert resp == {
            child.role_name,
            child.organization_account.role_name,
            child.account.role_name,
            parent.role_name,
        }



from uuid import uuid4

from psycopg2.extras import DictCursor

from oss_auth.database import DB
from oss_auth.utils.permission import Role
from tests.functional_tests.base import FunctionalBase
from tests.utils.factories.account import TokenFactory
from tests.utils.factories.base_factory import random_string
from tests.utils.factories.organization import OrganizationFactory


class OrganizationCreateTest(FunctionalBase):
    url = "/v1/organization/organization"

    def test_when_invalid_parameters__returns_422(self, test_app):
        orgs = [
            {"name": "valid", "active": "invalid"},
            {"name": "@invalid", "active": True},
            {"name": "valid"},
            {"active": "valid"},
        ]
        token = TokenFactory(account__role_name="superuser")
        DB._db.connection.commit()
        for org in orgs:
            self.assert_post(test_app, org, 400, token=token)

    def test_when_valid__returns_id_201(self, test_app):
        token = TokenFactory(account__role_name="superuser")
        org = {"name": "valid", "active": True}
        resp = self.assert_post(test_app, org, 201, token=token)
        created_org = DB.execute_query(
            "SELECT name, active FROM oss_auth.organization WHERE id = %(id)s",
            {"id": resp.json["id"]},
            factory=DictCursor,
        )[0]
        assert dict(created_org) == org


class OrganizationUpdateTest(FunctionalBase):
    url = "/v1/organization/organization/{}"

    def test_when_invalid_parameters__returns_422(self, test_app):
        org_model = OrganizationFactory()
        token = TokenFactory(account__role_name="superuser")
        DB._db.connection.commit()
        orgs = [
            {"name": "valid", "active": "invalid"},
            {"name": "@invalid", "active": True},
            {"name": "valid"},
            {"active": "valid"},
        ]
        for org in orgs:
            self.assert_put(test_app, org, 400, org_model.id, token=token)

    def test_when_no_org__returns_404(self, test_app):
        token = TokenFactory(account__role_name="superuser")
        fake_org_id = uuid4()
        self.assert_put(
            test_app,
            {"name": "valid", "active": True, "id": fake_org_id},
            404,
            fake_org_id,
            token=token,
        )

    def test_when_valid__returns_200(self, test_app):
        org = OrganizationFactory()
        token = TokenFactory(account__role_name="superuser")
        self.assert_put(
            test_app,
            {"name": "valid", "active": True, "id": org.id},
            200,
            org.id,
            token=token,
        )


class OrganizationGetAllTest(FunctionalBase):
    url = "/v1/organization/organization"

    def test_filter_orgs(self, test_app):
        name_part = random_string()
        OrganizationFactory(name=f"ab{name_part}")
        OrganizationFactory(name=f"b{name_part}e")
        OrganizationFactory(name="zxy")
        token = TokenFactory(account__role_name="superuser")
        query = f"name_filter={name_part}&page=0&page_size=3"
        resp = self.assert_get(test_app, {}, 200, token=token, query=query)
        assert len(resp.json) == 2


class OrganizationGetOrgTest(FunctionalBase):
    url = "/v1/organization/organization/{}"

    def test_when_existing__returns_org(self, test_app, superuser_org):
        resp = self.assert_get(
            test_app, {}, 200, superuser_org[1].organization_id, token=superuser_org[0]
        )
        assert resp.json["name"] == superuser_org[1].organization.name

    def test_when_not_exists__returns_404(self, test_app):
        token = TokenFactory(account__role_name=Role.superuser.value)
        self.assert_get(test_app, {}, 404, uuid4(), token=token)

    def test_when_exists_but_user_no_permission__returns_403(self, test_app):
        token = TokenFactory(account__role_name=Role.default.value)
        self.assert_get(test_app, {}, 403, uuid4(), token=token)

    def test_when_not_logged_in__returns_401(self, test_app):
        resp = self.assert_get(test_app, {}, 401, uuid4())
        assert "login_url" in resp.json

    def test_when_invalid_id_format__returns_400(self, test_app):
        token = TokenFactory(account__role_name=Role.default.value)
        self.assert_get(test_app, {}, 400, "blah", token=token)


class OrganizationNameAvailabilityTest(FunctionalBase):
    url = "/v1/organization/organization/name"

    def test_when_name_available__returns_success_200(self, test_app):
        query = "name=available"
        token = TokenFactory(account__role_name="superuser")
        resp = self.assert_get(test_app, {}, 200, token=token, query=query)
        assert resp.json["available"] is True

    def test_when_name_unavailable__returns_failure_200(self, test_app):
        OrganizationFactory(name="unavailable")
        token = TokenFactory(account__role_name="superuser")
        query = "name=unavailable"
        resp = self.assert_get(test_app, {}, 200, token=token, query=query)
        assert resp.json["available"] is False



# Local long term should only be used for things that very rarely change
# and where it's okay for users to get old data.
# Invalidating this cache really doesn't do much since other
# instances of the server will have a different cache
LOCAL_LONG_TERM = "local_long_term"



from logging import ERROR, WARNING

import pytest
from werkzeug.exceptions import HTTPException, ServiceUnavailable, UnprocessableEntity

from oss_auth.error_handlers import (
    _log_exception,
    handle_generic_exception,
    handle_http_exception,
)


class LogExceptionTest:
    def test_when_no_name__gets_class_name(self, log_capture, request_context):
        _log_exception(Exception("blah"))
        assert log_capture.records[1].args[1] == "Exception"

    def test_when_name__gets_name(self, log_capture, request_context):
        class Blah(Exception):
            name = "something"

        _log_exception(Blah("blah"))
        assert log_capture.records[1].args[1] == "something"

    def test_when_not_verbose__no_exception_log(self, log_capture, request_context):
        _log_exception(Exception("blah"), verbose=False)
        assert len(log_capture.records) == 1
        assert log_capture.records[0].levelno == WARNING

    def test_when_verbose__contains_exception_log(self, log_capture, request_context):
        _log_exception(Exception("blah"), verbose=True)
        assert len(log_capture.records) == 2
        assert log_capture.records[0].levelno == ERROR
        assert log_capture.records[1].levelno == WARNING


class HandleGenericExceptionTest:
    def test_when_valid__no_disclosed_info(self, request_context):
        resp = handle_generic_exception(Exception("blah"))
        assert resp.status_code == 500
        expected_response = {
            "message": "An Error has Occurred",
            "error_type": "ServerError",
        }
        assert resp.json == expected_response


class HandleHTTPExceptionTest:
    def test_when_5xx_exception__no_disclosed_info(self, request_context):
        resp = handle_http_exception(ServiceUnavailable("something something"))
        assert resp.status_code == ServiceUnavailable.code
        expected_response = {
            "message": "An Error has Occurred",
            "error_type": "ServerError",
        }
        assert resp.json == expected_response

    def test_when_4xx_exception__gives_reason(self, request_context):
        resp = handle_http_exception(UnprocessableEntity("something something"))
        assert resp.status_code == UnprocessableEntity.code
        expected_response = {
            "message": "something something",
            "error_type": "Unprocessable Entity",
        }
        assert resp.json == expected_response

    def test_when_3xx_exception__raises_exception(self, request_context):
        class Redirect(HTTPException):
            code = 302

        with pytest.raises(Redirect):
            handle_http_exception(Redirect("blah"))



from uuid import uuid4

import pytest
from werkzeug.routing import ValidationError

from oss_auth.utils.http import UUIDConverter


class UUIDConverterTest:
    def test_when_invalid__raises_validation_exception(self):
        converter = UUIDConverter(None)
        with pytest.raises(ValidationError):
            converter.to_python("invalid")

    def test_when_invalid_and_not_strict__raises_validation_exception(self):
        converter = UUIDConverter(None, strict=False)
        with pytest.raises(ValidationError):
            converter.to_python("invalid")

    def test_when_valid__returns_uuid(self):
        converter = UUIDConverter(None)
        expected = uuid4()
        resp = converter.to_python(str(expected))
        assert resp == expected

    def test_to_url__returns_str(self):
        converter = UUIDConverter(None)
        original = uuid4()
        resp = converter.to_url(original)
        assert resp == str(original)


from oss_auth.connexion_ext import UUIDParameterValidator


class ValidateParameterTest:
    def test_when_nullable_and_null__returns_empty(self):
        resp = UUIDParameterValidator.validate_parameter(
            "string", "null", {"schema": {"nullable": True, "type": "string"}}
        )
        assert resp is None



from uuid import uuid4

import pytest

from oss_auth.daos.account_dao import AccountDAO
from oss_auth.database import DB
from oss_auth.exceptions import NotLoggedInException
from oss_auth.services.authentication_service import (
    CognitoService,
    get_account_by_token,
    logout_account,
)
from tests.utils.conditionals import (
    skip_unless_cognito_tests_enabled,
    skip_unless_create_cognito_tests_enabled,
)
from tests.utils.factories.base_factory import random_email, random_string

from ...utils.factories.account import AccountFactory, TokenFactory


class LogoutAccountTest:
    def test_when_token_not_exists__no_error(self):
        logout_account("fake")

    def test_when_token_exists__deletes_token(self):
        token = TokenFactory()
        logout_account(token.token)
        resp = DB.execute_query(
            "SELECT COUNT(*) FROM oss_auth.token WHERE token = %s", (token.token,)
        )
        assert resp[0][0] == 0


class GetAccountByTokenTest:
    def test_when_no_token__raises_not_logged_in_exception(self):
        with pytest.raises(NotLoggedInException):
            get_account_by_token("fake")

    def test_when_token__returns_user_id(self):
        token = TokenFactory()
        resp = get_account_by_token(token.token)
        assert resp == token.account_id


class LoginUserTest:
    @skip_unless_cognito_tests_enabled
    def test_when_valid__returns_token(self):
        token = CognitoService.login_user("test@timmartin.me", "P@ssw0rd")
        assert _get_token(token).token == token


class RegisterUserTest:
    @skip_unless_create_cognito_tests_enabled
    def test_when_valid__creates_user(self):
        username = random_email()
        account = CognitoService.register_user(username, "P@ssw0rd")
        resp = AccountDAO.get_account_info(account["id"])
        assert resp.username == username


class GetOrCreateAccountTest:
    def test_when_existing__returns_existing_account(self):
        account = AccountFactory()
        account_info, created = CognitoService._get_or_create_account(
            account.cognito_id, account.email
        )
        assert created is False
        assert account_info["id"] == account.id

    def test_when_does_not_exist__creates_and_returns_account(self):
        cognito_id = uuid4()
        account_info, created = CognitoService._get_or_create_account(
            cognito_id, f"{random_string()}@{random_string()}"
        )
        assert created is True
        assert account_info["cognito_id"] == cognito_id


class CreateTokenTest:
    def test_when_expiration_is_none__sets_expiration_to_default_and_returns_token(
        self
    ):
        account = AccountFactory()
        resp = CognitoService._create_token(
            account.id, random_string(), random_string()
        )
        assert _get_token(resp)


def _get_token(token):
    return DB.execute_query(
        "SELECT * FROM token WHERE token.token = %s", (token,), expect_one=True
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

with open("README.md") as readme_file:
    readme = readme_file.read()

with open("HISTORY.md") as history_file:
    history = history_file.read()

with open("requirements.txt", "r") as requirements_file:
    requirements = requirements_file.read().splitlines()


setup(
    name="hou-flask",
    version="0.0.1",
    description="Basic authentication and authorization application",
    long_description=readme + "\n\n" + history,
    author="Tim Martin",
    author_email="oss@timmartin.me",
    url="https://github.com/timmartin19/hou-flask",
    packages=["houflask"],
    package_dir={"houflask": "houflask"},
    include_package_data=True,
    install_requires=requirements,
    zip_safe=False,
    keywords="houflask",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    test_suite="tests",
)



from oss_auth.daos.account_dao import AccountDAO
from oss_auth.utils.permission import Role
from tests.functional_tests.base import FunctionalBase
from tests.utils.factories import (
    AccountFactory,
    DepartmentAccountFactory,
    DepartmentFactory,
    OrganizationAccountFactory,
    OrganizationFactory,
    TokenFactory,
)
from tests.utils.factories.base_factory import random_string


class GetOrganizationsForAccountTest(FunctionalBase):
    url = "/v1/organization/account-self/organization-accounts"

    def test_when_valid__returns_organization_accounts(self, test_app):
        token = TokenFactory()
        OrganizationAccountFactory.build_batch(3, account=token.account)

        resp = self.assert_get(test_app, {}, 200, token=token)

        assert len(resp.json) == 3


class GetOrganizationForOwnAccountTest(FunctionalBase):
    url = "/v1/organization/account-self/organizations"

    def test_when_valid__return_organizations(self, test_app):
        token = TokenFactory()
        OrganizationAccountFactory.build_batch(3, account=token.account)

        resp = self.assert_get(test_app, {}, 200, token=token)

        assert len(resp.json) == 3


class UpsertAccountOrgTest(FunctionalBase):
    url = "/v1/organization/organization/{}/members/{}"

    def test_when_valid__adds_member(self, test_app, superuser_token):
        org = OrganizationFactory()
        account = AccountFactory()
        self.assert_put(
            test_app,
            {
                "account_id": account.id,
                "organization_id": org.id,
                "username": account.email,
                "role_name": Role.default.value,
            },
            200,
            org.id,
            account.id,
            token=superuser_token,
        )


class RemoveAccountOrgTest(FunctionalBase):
    url = "/v1/organization/organization/{}/members/{}"

    def test_when_valid__remove_account_org(self, test_app, superuser_token):
        org_account = OrganizationAccountFactory()
        self.assert_delete(
            test_app,
            {},
            204,
            org_account.organization_id,
            org_account.account_id,
            token=superuser_token,
        )
        rows = AccountDAO.get_orgs_for_account(org_account.account_id)
        assert rows == []


class GetUsernamesTest(FunctionalBase):
    url = "/v1/username"

    def test_when_valid__gets_usernames(self, test_app, superuser_token):
        account = AccountFactory()

        resp = self.assert_get(
            test_app,
            {"name_filter": str(account.email)[1:], "page": 0, "page_size": 10},
            200,
            token=superuser_token,
        )

        assert resp.json == [{"username": account.email, "account_id": str(account.id)}]
